#ifndef __ASN1_BLOB_H
#define __ASN1_BLOB_H

#ifdef _WIN32
#include <windows.h>
#include <WinCrypt.h>
#include <time.h>
#else
#include "CSP_WinCrypt.h"
#endif

#include <list>
#include "ASN1Exceptions.h"
#include <string.h>
namespace asn1data {
struct ASN1T_Name;
}

#ifdef ANDROID
namespace std {
	typedef basic_string<wchar_t> wstring;
}
#endif

// 4 Kb
#define CACMPT_BLOB_MEM_UNIT_SIZE ( 4 * 1024 )

struct CACMPT_BLOB {

    CACMPT_BLOB() : cbData(0), pbData(0), assignedSize( 0 )
    {
    }

    CACMPT_BLOB( const BYTE *ptr, DWORD l ) : cbData(0), pbData(0), assignedSize( 0 )
    {
	assign( ptr, l );
    }

    CACMPT_BLOB( const CACMPT_BLOB &src ) : cbData( 0 ), pbData( 0 ), assignedSize( 0 )
    {
	assign( src.pbData, src.cbData );
    }

    CACMPT_BLOB( DWORD l ) : cbData(0), pbData(0), assignedSize(0)
    {
	resize_to( l, FALSE );
    }

    ~CACMPT_BLOB()
    {
	delete[] pbData;
    }

    void resize_to( DWORD l, BOOL doCopy ) 
    { 
	if( l <= assignedSize )
	{
	    cbData = l;
	    return;
	}

	DWORD newAssignedSize = ( assignedSize == 0 ) ? CACMPT_BLOB_MEM_UNIT_SIZE : assignedSize;
	while( l > newAssignedSize )
	{
	    newAssignedSize *= 2;
	}

	BYTE *p = new BYTE[ newAssignedSize ];

	if( cbData && doCopy )
	    memcpy( p, pbData, cbData );
	delete[] pbData;
	pbData = p;
	cbData = l;
	assignedSize = newAssignedSize;
    }

    void resize_to( DWORD l ) 
    {
	resize_to( l, TRUE );
    }

    void set_size( DWORD l ) 
    {
	resize_to( l, FALSE );
    }

    void swap( CACMPT_BLOB &right ) throw()
    { 
	std::swap ( pbData, right.pbData );
	std::swap ( cbData, right.cbData );
    }

    void reverse()
    {
	for( DWORD i = 0; i < cbData / 2; i++ )
	{ 
	    BYTE tmp = pbData[i]; 
	    pbData[i] = pbData[cbData - i - 1];
	    pbData[cbData - i - 1] = tmp; 
	}
    }

    void append( const BYTE *ptr, DWORD l )
    { 
	resize_to( cbData + l );
	if( l )
	{
	    memcpy( pbData + cbData - l, ptr, l );
	}
    }

    CACMPT_BLOB& operator+=( const CACMPT_BLOB &right )
    { 
	resize_to( cbData + right.cbData );
	if( right.cbData )
	    memcpy( pbData + cbData - right.cbData, right.pbData, right.cbData );
	return *this;
    }

    void clear()
    {
	resize_to( 0 );
    }

    CACMPT_BLOB& operator=( const CACMPT_BLOB &src )
    {
	assign( src.pbData, src.cbData );
	return *this;
    }

    bool operator==( const CACMPT_BLOB &src ) const
    { return src.cbData == cbData && ( !cbData ||
	!memcmp( src.pbData, pbData, cbData ) ); }
    bool operator!=( const CACMPT_BLOB &src ) const
    { return !( *this == src ); }

    // CPCSP-5790
    bool operator<( const CACMPT_BLOB &src ) const
    { return cbData < src.cbData || (cbData == src.cbData && cbData
    && memcmp( pbData, src.pbData, cbData ) < 0); }
    bool operator<=( const CACMPT_BLOB &src ) const
    { return cbData < src.cbData || (cbData == src.cbData && cbData
    && memcmp( pbData, src.pbData, cbData ) <= 0); }
    bool operator>( const CACMPT_BLOB &src ) const
    { return cbData < src.cbData || (cbData == src.cbData && cbData
    && memcmp( pbData, src.pbData, cbData ) > 0); }
    bool operator>=( const CACMPT_BLOB &src ) const
    { return cbData < src.cbData || (cbData == src.cbData && cbData
    && memcmp( pbData, src.pbData, cbData ) >= 0); }
    int compare( const CACMPT_BLOB &src ) const
    { 
	if( cbData == src.cbData && cbData )
	    return memcmp( pbData, src.pbData, cbData );
	else
	    return cbData - src.cbData;
    }

    void assign( const BYTE *ptr, DWORD l )
    {
	if( l == 0 )
	{
	    delete[] pbData;
	    cbData = 0;
	    assignedSize = 0;
	    pbData = NULL;
	}
	else // l > 0
	{
	    resize_to( l );
	    memcpy( pbData, ptr, l );
	}
    }

    void readFromFile( const char *name );
    void readFromFile( const std::string &name ) { readFromFile( name.c_str() ); }
    void writeToFile( const char *name ) const;
    void writeToFile( const std::string &name ) const
    { writeToFile( name.c_str() ); }
    void readFromHexString( const char *str );
    std::string writeToHexString() const;
    CACMPT_BLOB toBase64() const;
    CACMPT_BLOB fromBase64() const;
    CACMPT_BLOB toBase64Hdr( const char *header, 
	const char *footer ) const;
    CACMPT_BLOB fromBase64Hdr( const char *header, 
	const char *footer ) const;

    static const char *Base64CertificateHeader;
    static const char *Base64CertificateFooter;
    static const char *Base64RequestHeader;
    static const char *Base64RequestFooter;
    static const char *Base64CRLHeader;
    static const char *Base64CRLFooter;
    DWORD cbData;
    BYTE *pbData;
    DWORD assignedSize;
};

typedef std::list<CACMPT_BLOB> blob_list;
typedef blob_list encoded_certificate_list;
typedef blob_list encoded_crl_list;

struct CACMPT_PublicKeyInfo {
    CACMPT_PublicKeyInfo();
    CACMPT_PublicKeyInfo( const CACMPT_PublicKeyInfo &src );

    //  -		ASN1T_SubjectPublicKeyInfo.ASN1T_AlgorithmIdentifier.ASN1TObjId algorithm
    char Algorithm[ 254 ];
    //  -	ASN1T_SubjectPublicKeyInfo.ASN1T_AlgorithmIdentifier.ASN1TOpenType parameters
    CACMPT_BLOB AlgorithmParameters;
    //  -	ASN1T_SubjectPublicKeyInfo.ASN1TDynBitStr subjectPublicKey
    CACMPT_BLOB PublicKey;

    bool operator==( const CACMPT_PublicKeyInfo &right ) const
    { return !strcmp( Algorithm, right.Algorithm )
	&& AlgorithmParameters == right.AlgorithmParameters
	&& PublicKey == right.PublicKey; }
    bool operator!= ( const CACMPT_PublicKeyInfo &right ) const
    { return !( *this == right ); }
    CACMPT_PublicKeyInfo &operator=( const CACMPT_PublicKeyInfo &src )
    { assign( src ); return *this; }
    void assign( const CACMPT_PublicKeyInfo &src );
};

struct CACMPT_AlgorithmIdentifier
{
    CACMPT_AlgorithmIdentifier();
    CACMPT_AlgorithmIdentifier( const CACMPT_AlgorithmIdentifier &src );

    //  -		ASN1T_AlgorithmIdentifier.ASN1TObjId algorithm
    char algorithm[ 254 ];
    //  -	ASN1T_AlgorithmIdentifier.ASN1TOpenType parameters
    CACMPT_BLOB parameters;

    bool operator==( const CACMPT_AlgorithmIdentifier &right ) const
    { return !strcmp( algorithm, right.algorithm )
	&& parameters == right.parameters; }
    bool operator!= ( const CACMPT_AlgorithmIdentifier &right ) const
    { return !( *this == right ); }
    CACMPT_AlgorithmIdentifier &operator=( const CACMPT_AlgorithmIdentifier &src )
    { assign( src ); return *this; }
    void assign( const CACMPT_AlgorithmIdentifier &src );
};

typedef std::string CACMPT_OID;

struct CACMPT_PARSED_RDN_OLD
{
    CACMPT_PARSED_RDN_OLD() {};
    CACMPT_PARSED_RDN_OLD &operator=( const CACMPT_PARSED_RDN_OLD &src )
    {
	assign( src );
	return *this;
    }
    CACMPT_PARSED_RDN_OLD( const CACMPT_PARSED_RDN_OLD& r );
    void clear()
    {
	CACMPT_PARSED_RDN_OLD tmp;
	swap( tmp );
    }
    void assign( const CACMPT_PARSED_RDN_OLD &src )
    {
	CACMPT_PARSED_RDN_OLD tmp( src );
	swap( tmp );
    }
    void swap( CACMPT_PARSED_RDN_OLD& r );

    // wide = teletexString, printableString, universalString, utf8String, bmpString
    std::string EMail; // EmailAddress pkcs-9 1 IA5String
    std::string Country; // X520countryName id-at 6 PrintableString
    std::wstring State; // X520StateOrProvinceName id-at 8  wide
    std::wstring Locality; // X520LocalityName id-at 7 wide
    std::wstring StreetAddress; // ??? id-at 9 ??? 
//    std::string UnstructuredAddress; // ??? RSA: 1.2.840.113549.1.9.8 ???
    std::wstring Organization; // X520OrganizationName id-at 10 wide
    std::wstring OrgUnit; // X520OrganizationalUnitName id-at 11 wide
    std::wstring Title; // X520Title id-at 12 wide
    // std::string DomainComponent; // DomainComponent 0.9.2342.19200300.100.1.25 IA5String
    // std::string DeviceSerialNumber; // X520SerialNumber id-at 5 PrintableString
    // temporary
    std::wstring GivenName; // X520name id-at 42 wide
    std::wstring Initials; // X520name id-at 41 wide
    std::wstring SurName; // X520name id-at 4 wide
    std::wstring Pseudonym; // X520Pseudonym id-at 65 wide
    std::wstring CommonName; // X520CommonName id-at 3 wide
    std::string OGRN; // OGRN
    std::string OGRNIP; // OGRNIP
    std::string INN; // INN
    std::string SNILS; // SNILS
    std::wstring UnstructuredName; // UnstructuredName
    std::wstring PostalCode; 
};

class CACMPT_PARSED_RDN
{
  protected:
    CACMPT_BLOB der;

    static const BYTE m_empty_der [];
    static const DWORD m_empty_der_len;
  public:
    CACMPT_PARSED_RDN () : der (m_empty_der, m_empty_der_len) { }
    CACMPT_PARSED_RDN (const CACMPT_BLOB &src) : der (src) {}
    CACMPT_PARSED_RDN (const struct asn1data::ASN1T_Name &src) { assign (src); }
    CACMPT_PARSED_RDN (const CACMPT_PARSED_RDN_OLD &src) { assign (src); }
    CACMPT_PARSED_RDN (const std::wstring& src) { assign (src); }

    CACMPT_PARSED_RDN &operator=( const CACMPT_PARSED_RDN &src );

    inline void clear () { der.assign (m_empty_der, m_empty_der_len); }
    void assign (const CACMPT_BLOB &src) { der = src; }
    void assign (const struct asn1data::ASN1T_Name &src);
    void assign (const CACMPT_PARSED_RDN_OLD &src);
    void assign (const std::wstring &src);

    std::wstring tostring (DWORD dwStrType = CERT_X500_NAME_STR|CERT_NAME_STR_NO_QUOTING_FLAG ) const;
    void parse (CACMPT_PARSED_RDN_OLD& old) const;
    void parse (class CACMPT_ASN1BERDecodeBuffer& dec_buffer, asn1data::ASN1T_Name& name) const;
};

typedef char	CACMPT_SERIAL [128];

struct CACMPT_BigInteger : public CACMPT_BLOB
{
public:
    CACMPT_BigInteger() : CACMPT_BLOB() { }
    CACMPT_BigInteger( const unsigned char *ptr, unsigned long l ) 
	: CACMPT_BLOB() { assign( ptr, l );  }
    CACMPT_BigInteger( const CACMPT_BigInteger &src )
	: CACMPT_BLOB() { assign( src.pbData, src.cbData );  }
    CACMPT_BigInteger( unsigned long l ) 
	: CACMPT_BLOB(l) { }

    void increment( int start);
    CACMPT_BigInteger& operator++();
    CACMPT_BigInteger operator++(int);
};

#endif // __ASN1_BLOB_H
